@@ -1,52 +1,66 @@
 import json
 from re import match
-from functools import reduce
-from typing import Any, Dict, Optional
+from typing import Any, Dict, List, Optional
 
 from samtranslator.model import PropertyType, Resource
 from samtranslator.model.exceptions import InvalidResourceException
-from samtranslator.model.types import is_type, one_of, is_str, list_of
-from samtranslator.model.intrinsics import ref, fnSub
+from samtranslator.model.intrinsics import fnSub, ref
+from samtranslator.model.types import IS_DICT, IS_STR, is_type, list_of, one_of
+from samtranslator.schema.common import PassThrough
 from samtranslator.translator import logical_id_generator
 from samtranslator.translator.arn_generator import ArnGenerator
 from samtranslator.utils.py27hash_fix import Py27Dict, Py27UniStr
+from samtranslator.validator.value_validator import sam_expect
 
 
 class ApiGatewayRestApi(Resource):
     resource_type = "AWS::ApiGateway::RestApi"
     property_types = {
-        "Body": PropertyType(False, is_type(dict)),
-        "BodyS3Location": PropertyType(False, is_type(dict)),
-        "CloneFrom": PropertyType(False, is_str()),
-        "Description": PropertyType(False, is_str()),
+        "Body": PropertyType(False, IS_DICT),
+        "BodyS3Location": PropertyType(False, IS_DICT),
+        "CloneFrom": PropertyType(False, IS_STR),
+        "Description": PropertyType(False, IS_STR),
         "FailOnWarnings": PropertyType(False, is_type(bool)),
-        "Name": PropertyType(False, is_str()),
-        "Parameters": PropertyType(False, is_type(dict)),
-        "EndpointConfiguration": PropertyType(False, is_type(dict)),
+        "Name": PropertyType(False, IS_STR),
+        "Parameters": PropertyType(False, IS_DICT),
+        "EndpointConfiguration": PropertyType(False, IS_DICT),
         "BinaryMediaTypes": PropertyType(False, is_type(list)),
         "MinimumCompressionSize": PropertyType(False, is_type(int)),
-        "Mode": PropertyType(False, is_str()),
-        "ApiKeySourceType": PropertyType(False, is_str()),
+        "Mode": PropertyType(False, IS_STR),
+        "ApiKeySourceType": PropertyType(False, IS_STR),
     }
 
+    Body: Optional[Dict[str, Any]]
+    BodyS3Location: Optional[Dict[str, Any]]
+    CloneFrom: Optional[PassThrough]
+    Description: Optional[PassThrough]
+    FailOnWarnings: Optional[PassThrough]
+    Name: Optional[PassThrough]
+    Parameters: Optional[Dict[str, Any]]
+    EndpointConfiguration: Optional[Dict[str, Any]]
+    BinaryMediaTypes: Optional[List[Any]]
+    MinimumCompressionSize: Optional[PassThrough]
+    Mode: Optional[PassThrough]
+    ApiKeySourceType: Optional[PassThrough]
+
     runtime_attrs = {"rest_api_id": lambda self: ref(self.logical_id)}
 
 
 class ApiGatewayStage(Resource):
     resource_type = "AWS::ApiGateway::Stage"
     property_types = {
-        "AccessLogSetting": PropertyType(False, is_type(dict)),
+        "AccessLogSetting": PropertyType(False, IS_DICT),
         "CacheClusterEnabled": PropertyType(False, is_type(bool)),
-        "CacheClusterSize": PropertyType(False, is_str()),
-        "CanarySetting": PropertyType(False, is_type(dict)),
-        "ClientCertificateId": PropertyType(False, is_str()),
-        "DeploymentId": PropertyType(True, is_str()),
-        "Description": PropertyType(False, is_str()),
-        "RestApiId": PropertyType(True, is_str()),
-        "StageName": PropertyType(True, one_of(is_str(), is_type(dict))),
-        "Tags": PropertyType(False, list_of(is_type(dict))),
+        "CacheClusterSize": PropertyType(False, IS_STR),
+        "CanarySetting": PropertyType(False, IS_DICT),
+        "ClientCertificateId": PropertyType(False, IS_STR),
+        "DeploymentId": PropertyType(True, IS_STR),
+        "Description": PropertyType(False, IS_STR),
+        "RestApiId": PropertyType(True, IS_STR),
+        "StageName": PropertyType(True, one_of(IS_STR, IS_DICT)),
+        "Tags": PropertyType(False, list_of(IS_DICT)),
         "TracingEnabled": PropertyType(False, is_type(bool)),
-        "Variables": PropertyType(False, is_type(dict)),
+        "Variables": PropertyType(False, IS_DICT),
         "MethodSettings": PropertyType(False, is_type(list)),
     }
 
@@ -58,7 +72,7 @@ class ApiGatewayStage(Resource):
 
 class ApiGatewayAccount(Resource):
     resource_type = "AWS::ApiGateway::Account"
-    property_types = {"CloudWatchRoleArn": PropertyType(False, one_of(is_str(), is_type(dict)))}
+    property_types = {"CloudWatchRoleArn": PropertyType(False, one_of(IS_STR, IS_DICT))}
 
 
 class ApiGatewayDeployment(Resource):
@@ -66,10 +80,10 @@ class ApiGatewayDeployment(Resource):
 
     resource_type = "AWS::ApiGateway::Deployment"
     property_types = {
-        "Description": PropertyType(False, is_str()),
-        "RestApiId": PropertyType(True, is_str()),
-        "StageDescription": PropertyType(False, is_type(dict)),
-        "StageName": PropertyType(False, is_str()),
+        "Description": PropertyType(False, IS_STR),
+        "RestApiId": PropertyType(True, IS_STR),
+        "StageDescription": PropertyType(False, IS_DICT),
+        "StageName": PropertyType(False, IS_STR),
     }
 
     runtime_attrs = {"deployment_id": lambda self: ref(self.logical_id)}
@@ -123,10 +137,11 @@ class ApiGatewayResponse(object):
         self,
         api_logical_id: str,
         response_parameters: Optional[Dict[str, Any]] = None,
-        response_templates: Optional[Dict[str, Any]] = None,
+        response_templates: Optional[PassThrough] = None,
         status_code: Optional[str] = None,
     ) -> None:
         if response_parameters:
+            # response_parameters has been validated in ApiGenerator._add_gateway_responses()
             for response_parameter_key in response_parameters.keys():
                 if response_parameter_key not in ApiGatewayResponse.ResponseParameterProperties:
                     raise InvalidResourceException(
@@ -162,8 +177,12 @@ class ApiGatewayResponse(object):
         prefixed_parameters = Py27Dict()
 
         parameter_prefix_pairs = [("Headers", "header."), ("Paths", "path."), ("QueryStrings", "querystring.")]
-        for parameter, prefix in parameter_prefix_pairs:
-            for key, value in response_parameters.get(parameter, {}).items():
+        for parameter_property_name, prefix in parameter_prefix_pairs:
+            parameter_property_value = response_parameters.get(parameter_property_name, {})
+            sam_expect(
+                parameter_property_value, self.api_logical_id, f"ResponseParameters.{parameter_property_name}"
+            ).to_be_a_map()
+            for key, value in parameter_property_value.items():
                 param_key = GATEWAY_RESPONSE_PREFIX + prefix + key
                 if isinstance(key, Py27UniStr):
                     # if key is from template, we need to convert param_key to Py27UniStr
@@ -179,23 +198,23 @@ class ApiGatewayResponse(object):
 class ApiGatewayDomainName(Resource):
     resource_type = "AWS::ApiGateway::DomainName"
     property_types = {
-        "RegionalCertificateArn": PropertyType(False, is_str()),
-        "DomainName": PropertyType(True, is_str()),
-        "EndpointConfiguration": PropertyType(False, is_type(dict)),
-        "MutualTlsAuthentication": PropertyType(False, is_type(dict)),
-        "SecurityPolicy": PropertyType(False, is_str()),
-        "CertificateArn": PropertyType(False, is_str()),
-        "OwnershipVerificationCertificateArn": PropertyType(False, is_str()),
+        "RegionalCertificateArn": PropertyType(False, IS_STR),
+        "DomainName": PropertyType(True, IS_STR),
+        "EndpointConfiguration": PropertyType(False, IS_DICT),
+        "MutualTlsAuthentication": PropertyType(False, IS_DICT),
+        "SecurityPolicy": PropertyType(False, IS_STR),
+        "CertificateArn": PropertyType(False, IS_STR),
+        "OwnershipVerificationCertificateArn": PropertyType(False, IS_STR),
     }
 
 
 class ApiGatewayBasePathMapping(Resource):
     resource_type = "AWS::ApiGateway::BasePathMapping"
     property_types = {
-        "BasePath": PropertyType(False, is_str()),
-        "DomainName": PropertyType(True, is_str()),
-        "RestApiId": PropertyType(False, is_str()),
-        "Stage": PropertyType(False, is_str()),
+        "BasePath": PropertyType(False, IS_STR),
+        "DomainName": PropertyType(True, IS_STR),
+        "RestApiId": PropertyType(False, IS_STR),
+        "Stage": PropertyType(False, IS_STR),
     }
 
 
@@ -203,11 +222,11 @@ class ApiGatewayUsagePlan(Resource):
     resource_type = "AWS::ApiGateway::UsagePlan"
     property_types = {
         "ApiStages": PropertyType(False, is_type(list)),
-        "Description": PropertyType(False, is_str()),
-        "Quota": PropertyType(False, is_type(dict)),
+        "Description": PropertyType(False, IS_STR),
+        "Quota": PropertyType(False, IS_DICT),
         "Tags": PropertyType(False, list_of(dict)),
-        "Throttle": PropertyType(False, is_type(dict)),
-        "UsagePlanName": PropertyType(False, is_str()),
+        "Throttle": PropertyType(False, IS_DICT),
+        "UsagePlanName": PropertyType(False, IS_STR),
     }
     runtime_attrs = {"usage_plan_id": lambda self: ref(self.logical_id)}
 
@@ -215,22 +234,22 @@ class ApiGatewayUsagePlan(Resource):
 class ApiGatewayUsagePlanKey(Resource):
     resource_type = "AWS::ApiGateway::UsagePlanKey"
     property_types = {
-        "KeyId": PropertyType(True, is_str()),
-        "KeyType": PropertyType(True, is_str()),
-        "UsagePlanId": PropertyType(True, is_str()),
+        "KeyId": PropertyType(True, IS_STR),
+        "KeyType": PropertyType(True, IS_STR),
+        "UsagePlanId": PropertyType(True, IS_STR),
     }
 
 
 class ApiGatewayApiKey(Resource):
     resource_type = "AWS::ApiGateway::ApiKey"
     property_types = {
-        "CustomerId": PropertyType(False, is_str()),
-        "Description": PropertyType(False, is_str()),
+        "CustomerId": PropertyType(False, IS_STR),
+        "Description": PropertyType(False, IS_STR),
         "Enabled": PropertyType(False, is_type(bool)),
         "GenerateDistinctId": PropertyType(False, is_type(bool)),
-        "Name": PropertyType(False, is_str()),
+        "Name": PropertyType(False, IS_STR),
         "StageKeys": PropertyType(False, is_type(list)),
-        "Value": PropertyType(False, is_str()),
+        "Value": PropertyType(False, IS_STR),
     }
 
     runtime_attrs = {"api_key_id": lambda self: ref(self.logical_id)}
@@ -253,21 +272,6 @@ class ApiGatewayAuthorizer(object):
     ):
         if authorization_scopes is None:
             authorization_scopes = []
-        if function_payload_type not in ApiGatewayAuthorizer._VALID_FUNCTION_PAYLOAD_TYPES:
-            raise InvalidResourceException(
-                api_logical_id,
-                f"{name} Authorizer has invalid 'FunctionPayloadType': {function_payload_type}.",
-            )
-
-        if function_payload_type == "REQUEST" and self._is_missing_identity_source(identity):  # type: ignore[no-untyped-call]
-            raise InvalidResourceException(
-                api_logical_id,
-                f"{name} Authorizer must specify Identity with at least one "
-                "of Headers, QueryStrings, StageVariables, or Context.",
-            )
-
-        if authorization_scopes is not None and not isinstance(authorization_scopes, list):
-            raise InvalidResourceException(api_logical_id, "AuthorizationScopes must be a list.")
 
         self.api_logical_id = api_logical_id
         self.name = name
@@ -279,10 +283,28 @@ class ApiGatewayAuthorizer(object):
         self.is_aws_iam_authorizer = is_aws_iam_authorizer
         self.authorization_scopes = authorization_scopes
 
-    def _is_missing_identity_source(self, identity):  # type: ignore[no-untyped-def]
+        if function_payload_type not in ApiGatewayAuthorizer._VALID_FUNCTION_PAYLOAD_TYPES:
+            raise InvalidResourceException(
+                api_logical_id,
+                f"{name} Authorizer has invalid 'FunctionPayloadType': {function_payload_type}.",
+            )
+
+        if function_payload_type == "REQUEST" and self._is_missing_identity_source(identity):
+            raise InvalidResourceException(
+                api_logical_id,
+                f"{name} Authorizer must specify Identity with at least one "
+                "of Headers, QueryStrings, StageVariables, or Context.",
+            )
+
+        if authorization_scopes is not None and not isinstance(authorization_scopes, list):
+            raise InvalidResourceException(api_logical_id, "AuthorizationScopes must be a list.")
+
+    def _is_missing_identity_source(self, identity: Dict[str, Any]) -> bool:
         if not identity:
             return True
 
+        sam_expect(identity, self.api_logical_id, f"Authorizer.{self.name}.Identity").to_be_a_map()
+
         headers = identity.get("Headers")
         query_strings = identity.get("QueryStrings")
         stage_variables = identity.get("StageVariables")
@@ -291,9 +313,11 @@ class ApiGatewayAuthorizer(object):
 
         required_properties_missing = not headers and not query_strings and not stage_variables and not context
 
+        if ttl is None:
+            return required_properties_missing
         try:
             ttl_int = int(ttl)
-        # this will catch if ttl is None and not convertable to an int
+        # this will catch if and not convertable to an int
         except (TypeError, ValueError):
             # previous behavior before trying to read ttl
             return required_properties_missing
@@ -318,7 +342,7 @@ class ApiGatewayAuthorizer(object):
 
         elif authorizer_type == "LAMBDA":
             swagger[APIGATEWAY_AUTHORIZER_KEY] = Py27Dict({"type": self._get_swagger_authorizer_type()})  # type: ignore[no-untyped-call, no-untyped-call]
-            partition = ArnGenerator.get_partition_name()  # type: ignore[no-untyped-call]
+            partition = ArnGenerator.get_partition_name()
             resource = "lambda:path/2015-03-31/functions/${__FunctionArn__}/invocations"
             authorizer_uri = fnSub(
                 ArnGenerator.generate_arn(  # type: ignore[no-untyped-call]
@@ -338,9 +362,9 @@ class ApiGatewayAuthorizer(object):
                 swagger[APIGATEWAY_AUTHORIZER_KEY]["authorizerCredentials"] = function_invoke_role
 
             if self._get_function_payload_type() == "REQUEST":  # type: ignore[no-untyped-call]
-                identity_source = self._get_identity_source()  # type: ignore[no-untyped-call]
+                identity_source = self._get_identity_source()
                 if identity_source:
-                    swagger[APIGATEWAY_AUTHORIZER_KEY]["identitySource"] = self._get_identity_source()  # type: ignore[no-untyped-call]
+                    swagger[APIGATEWAY_AUTHORIZER_KEY]["identitySource"] = self._get_identity_source()
 
         # Authorizer Validation Expression is only allowed on COGNITO_USER_POOLS and LAMBDA_TOKEN
         is_lambda_token_authorizer = authorizer_type == "LAMBDA" and self._get_function_payload_type() == "TOKEN"  # type: ignore[no-untyped-call]
@@ -356,21 +380,25 @@ class ApiGatewayAuthorizer(object):
     def _get_identity_validation_expression(self):  # type: ignore[no-untyped-def]
         return self.identity and self.identity.get("ValidationExpression")
 
-    def _build_identity_source_item(self, item_prefix, prop_value):  # type: ignore[no-untyped-def]
+    @staticmethod
+    def _build_identity_source_item(item_prefix: str, prop_value: str) -> str:
         item = item_prefix + prop_value
         if isinstance(prop_value, Py27UniStr):
             item = Py27UniStr(item)
         return item
 
-    def _build_identity_source_item_array(self, prop_key, item_prefix):  # type: ignore[no-untyped-def]
-        arr = []
-        if self.identity.get(prop_key):
-            arr = [
-                self._build_identity_source_item(item_prefix, prop_value) for prop_value in self.identity.get(prop_key)  # type: ignore[no-untyped-call]
-            ]
+    def _build_identity_source_item_array(self, prop_key: str, item_prefix: str) -> List[str]:
+        arr: List[str] = []
+        prop_value_list = self.identity.get(prop_key)
+        if prop_value_list:
+            prop_path = f"Auth.Authorizers.{self.name}.Identity.{prop_key}"
+            sam_expect(prop_value_list, self.api_logical_id, prop_path).to_be_a_list()
+            for index, prop_value in enumerate(prop_value_list):
+                sam_expect(prop_value, self.api_logical_id, f"{prop_path}[{index}]").to_be_a_string()
+                arr.append(self._build_identity_source_item(item_prefix, prop_value))
         return arr
 
-    def _get_identity_source(self):  # type: ignore[no-untyped-def]
+    def _get_identity_source(self) -> str:
         key_prefix_pairs = [
             ("Headers", "method.request.header."),
             ("QueryStrings", "method.request.querystring."),
@@ -378,12 +406,9 @@ class ApiGatewayAuthorizer(object):
             ("Context", "context."),
         ]
 
-        identity_source_array = reduce(  # type: ignore[var-annotated]
-            lambda accumulator, key_prefix_pair: accumulator  # type: ignore[no-any-return]
-            + self._build_identity_source_item_array(key_prefix_pair[0], key_prefix_pair[1]),  # type: ignore[no-untyped-call]
-            key_prefix_pairs,
-            [],
-        )
+        identity_source_array = []
+        for prop_key, item_prefix in key_prefix_pairs:
+            identity_source_array.extend(self._build_identity_source_item_array(prop_key, item_prefix))
 
         identity_source = ", ".join(identity_source_array)
         if any(isinstance(i, Py27UniStr) for i in identity_source_array):
