@@ -1,7 +1,15 @@
 from abc import ABC, abstractmethod
+from enum import Enum
 from typing import List, Optional, Sequence, Union
 
 
+class ExpectedType(Enum):
+    MAP = ("map", dict)
+    LIST = ("list", list)
+    STRING = ("string", str)
+    INTEGER = ("integer", int)
+
+
 class ExceptionWithMessage(ABC, Exception):
     @property
     @abstractmethod
@@ -18,7 +26,10 @@ class InvalidDocumentException(ExceptionWithMessage):
     """
 
     def __init__(self, causes: Sequence[ExceptionWithMessage]):
-        self._causes = causes
+        self._causes = list(causes)
+        # Sometimes, the same error could be raised from different plugins,
+        # so here we do a deduplicate based on the message:
+        self._causes = list({cause.message: cause for cause in self._causes}.values())
 
     @property
     def message(self) -> str:
@@ -37,7 +48,7 @@ class DuplicateLogicalIdException(ExceptionWithMessage):
         message -- explanation of the error
     """
 
-    def __init__(self, logical_id, duplicate_id, type):  # type: ignore[no-untyped-def]
+    def __init__(self, logical_id: str, duplicate_id: str, type: str) -> None:
         self._logical_id = logical_id
         self._duplicate_id = duplicate_id
         self._type = type
@@ -87,6 +98,27 @@ class InvalidResourceException(ExceptionWithMessage):
         return "Resource with id [{}] is invalid. {}".format(self._logical_id, self._message)
 
 
+class InvalidResourcePropertyTypeException(InvalidResourceException):
+    def __init__(
+        self,
+        logical_id: str,
+        property_identifier: str,
+        expected_type: Optional[ExpectedType],
+        message: Optional[str] = None,
+    ) -> None:
+        message = message or self._default_message(property_identifier, expected_type)
+        super().__init__(logical_id, message)
+
+        self.property_identifier = property_identifier
+
+    @staticmethod
+    def _default_message(property_identifier: str, expected_type: Optional[ExpectedType]) -> str:
+        if expected_type:
+            type_description, _ = expected_type.value
+            return f"Property '{property_identifier}' should be a {type_description}."
+        return f"Type of property '{property_identifier}' is invalid."
+
+
 class InvalidEventException(ExceptionWithMessage):
     """Exception raised when an event is invalid.
 
