@@ -1,15 +1,15 @@
 from typing import Any, List, Optional
 
 from samtranslator.model import PropertyType, Resource
-from samtranslator.model.types import is_type, is_str
+from samtranslator.model.types import is_type, IS_STR
 from samtranslator.utils.types import Intrinsicable
 
 
 class Route53RecordSetGroup(Resource):
     resource_type = "AWS::Route53::RecordSetGroup"
     property_types = {
-        "HostedZoneId": PropertyType(False, is_str()),
-        "HostedZoneName": PropertyType(False, is_str()),
+        "HostedZoneId": PropertyType(False, IS_STR),
+        "HostedZoneName": PropertyType(False, IS_STR),
         "RecordSets": PropertyType(False, is_type(list)),
     }
 
