@@ -1,8 +1,9 @@
+from typing import Any, Dict, Optional, Union
 from urllib.parse import urlparse, parse_qs
 from samtranslator.model.exceptions import InvalidResourceException
 
 
-def parse_s3_uri(uri):
+def parse_s3_uri(uri: Any) -> Optional[Dict[str, Any]]:
     """Parses a S3 Uri into a dictionary of the Bucket, Key, and VersionId
 
     :return: a BodyS3Location dict or None if not an S3 Uri
@@ -19,11 +20,10 @@ def parse_s3_uri(uri):
         if "versionId" in query and len(query["versionId"]) == 1:
             s3_pointer["Version"] = query["versionId"][0]
         return s3_pointer
-    else:
-        return None
+    return None
 
 
-def to_s3_uri(code_dict):
+def to_s3_uri(code_dict):  # type: ignore[no-untyped-def]
     """Constructs a S3 URI string from given code dictionary
 
     :param dict code_dict: Dictionary containing Lambda function Code S3 location of the form
@@ -44,7 +44,7 @@ def to_s3_uri(code_dict):
     return uri
 
 
-def construct_image_code_object(image_uri, logical_id, property_name):
+def construct_image_code_object(image_uri, logical_id, property_name):  # type: ignore[no-untyped-def]
     """Constructs a Lambda `Code` or `Content` property, from the SAM `ImageUri` property.
     This follows the current scheme for Lambda Functions.
 
@@ -62,7 +62,9 @@ def construct_image_code_object(image_uri, logical_id, property_name):
     return {"ImageUri": image_uri}
 
 
-def construct_s3_location_object(location_uri, logical_id, property_name):
+def construct_s3_location_object(
+    location_uri: Union[str, Dict[str, Any]], logical_id: str, property_name: str
+) -> Dict[str, Any]:
     """Constructs a Lambda `Code` or `Content` property, from the SAM `CodeUri` or `ContentUri` property.
     This follows the current scheme for Lambda Functions and LayerVersions.
 
@@ -83,15 +85,16 @@ def construct_s3_location_object(location_uri, logical_id, property_name):
 
     else:
         # location_uri is NOT a dictionary. Parse it as a string
-        s3_pointer = parse_s3_uri(location_uri)
+        _s3_pointer = parse_s3_uri(location_uri)
 
-        if s3_pointer is None:
+        if _s3_pointer is None:
             raise InvalidResourceException(
                 logical_id,
                 "'{}' is not a valid S3 Uri of the form "
                 "'s3://bucket/key' with optional versionId query "
                 "parameter.".format(property_name),
             )
+        s3_pointer = _s3_pointer
 
     code = {"S3Bucket": s3_pointer["Bucket"], "S3Key": s3_pointer["Key"]}
     if "Version" in s3_pointer:
