@@ -21,6 +21,9 @@ class StateMachineGenerator(object):
     _SAM_VALUE = "SAM"
     _SUBSTITUTION_NAME_TEMPLATE = "definition_substitution_%s"
     _SUBSTITUTION_KEY_TEMPLATE = "${definition_substitution_%s}"
+    SFN_INVALID_PROPERTY_BOTH_ROLE_POLICY = (
+        "Specify either 'Role' or 'Policies' (but not both at the same time) or neither of them"
+    )
 
     def __init__(  # type: ignore[no-untyped-def]
         self,
@@ -41,6 +44,7 @@ class StateMachineGenerator(object):
         events,
         event_resources,
         event_resolver,
+        role_path=None,
         tags=None,
         resource_attributes=None,
         passthrough_resource_attributes=None,
@@ -60,6 +64,7 @@ class StateMachineGenerator(object):
         :param permissions_boundary: The ARN of the policy used to set the permissions boundary for the role
         :param definition_substitutions: Variable-to-value mappings to be replaced in the State Machine definition
         :param role: Role ARN to use for the execution role
+        :param role_path: The file path of the execution role
         :param state_machine_type: Type of the State Machine
         :param tracing: Tracing configuration for the State Machine
         :param events: List of event sources for the State Machine
@@ -83,6 +88,7 @@ class StateMachineGenerator(object):
         self.permissions_boundary = permissions_boundary
         self.definition_substitutions = definition_substitutions
         self.role = role
+        self.role_path = role_path
         self.type = state_machine_type
         self.tracing = tracing
         self.events = events
@@ -129,20 +135,17 @@ class StateMachineGenerator(object):
             )
 
         if self.role and self.policies:
-            raise InvalidResourceException(
-                self.logical_id, "Specify either 'Role' or 'Policies' property and not both."
-            )
+            raise InvalidResourceException(self.logical_id, self.SFN_INVALID_PROPERTY_BOTH_ROLE_POLICY)
         if self.role:
             self.state_machine.RoleArn = self.role
-        elif self.policies:
-            if not self.managed_policy_map:
+        else:
+            if self.policies and not self.managed_policy_map:
                 raise Exception("Managed policy map is empty, but should not be.")
-
+            if not self.policies:
+                self.policies = []
             execution_role = self._construct_role()  # type: ignore[no-untyped-call]
             self.state_machine.RoleArn = execution_role.get_runtime_attr("arn")
             resources.append(execution_role)
-        else:
-            raise InvalidResourceException(self.logical_id, "Either 'Role' or 'Policies' property must be specified.")
 
         self.state_machine.StateMachineName = self.name
         self.state_machine.StateMachineType = self.type
@@ -210,9 +213,9 @@ class StateMachineGenerator(object):
         """
         policies = self.policies[:]
         if self.tracing and self.tracing.get("Enabled") is True:
-            policies.append(get_xray_managed_policy_name())  # type: ignore[no-untyped-call]
+            policies.append(get_xray_managed_policy_name())
 
-        state_machine_policies = ResourcePolicies(  # type: ignore[no-untyped-call]
+        state_machine_policies = ResourcePolicies(
             {"Policies": policies},
             # No support for policy templates in the "core"
             policy_template_processor=None,
@@ -220,6 +223,7 @@ class StateMachineGenerator(object):
 
         execution_role = construct_role_for_resource(
             resource_logical_id=self.logical_id,
+            role_path=self.role_path,
             attributes=self.passthrough_resource_attributes,
             managed_policy_map=self.managed_policy_map,
             assume_role_policy_document=IAMRolePolicies.stepfunctions_assume_role_policy(),  # type: ignore[no-untyped-call]
