@@ -1,5 +1,8 @@
+from typing import Any, Tuple
+
 import boto3
 import json
+from botocore.config import Config
 from botocore.exceptions import ClientError, EndpointConnectionError
 import logging
 from time import sleep
@@ -14,6 +17,8 @@ from samtranslator.public.sdk.template import SamTemplate
 from samtranslator.intrinsics.resolver import IntrinsicsResolver
 from samtranslator.intrinsics.actions import FindInMapAction
 from samtranslator.region_configuration import RegionConfiguration
+from samtranslator.utils.constants import BOTO3_CONNECT_TIMEOUT
+from samtranslator.validator.value_validator import sam_expect
 
 LOG = logging.getLogger(__name__)
 
@@ -70,6 +75,11 @@ class ServerlessAppPlugin(BasePlugin):
             message = "Cannot set both validate_only and wait_for_template_active_status flags to True."
             raise InvalidPluginException(ServerlessAppPlugin.__name__, message)  # type: ignore[no-untyped-call]
 
+    @staticmethod
+    def _make_app_key(app_id: Any, semver: Any) -> Tuple[str, str]:
+        """Generate a key that is always hashable."""
+        return json.dumps(app_id, default=str), json.dumps(semver, default=str)
+
     @cw_timer(prefix=PLUGIN_METRICS_PREFIX)
     def on_before_transform_template(self, template_dict):  # type: ignore[no-untyped-def]
         """
@@ -103,22 +113,27 @@ class ServerlessAppPlugin(BasePlugin):
                 app.properties[self.LOCATION_KEY], self.SEMANTIC_VERSION_KEY, intrinsic_resolvers
             )
 
+            key = self._make_app_key(app_id, semver)
+
             if isinstance(app_id, dict) or isinstance(semver, dict):
-                key = (json.dumps(app_id), json.dumps(semver))
                 self._applications[key] = False
                 continue
 
-            key = (app_id, semver)
-
             if key not in self._applications:
                 try:
+                    # Examine the type of ApplicationId and SemanticVersion
+                    # before calling SAR API.
+                    sam_expect(app_id, logical_id, "Location.ApplicationId").to_be_a_string()
+                    sam_expect(semver, logical_id, "Location.SemanticVersion").to_be_a_string()
                     if not RegionConfiguration.is_service_supported("serverlessrepo"):  # type: ignore[no-untyped-call]
                         raise InvalidResourceException(
                             logical_id, "Serverless Application Repository is not available in this region."
                         )
                     # Lazy initialization of the client- create it when it is needed
                     if not self._sar_client:
-                        self._sar_client = boto3.client("serverlessrepo")
+                        # a SAR call could take a while to finish, leaving the read_timeout default (60s).
+                        client_config = Config(connect_timeout=BOTO3_CONNECT_TIMEOUT)
+                        self._sar_client = boto3.client("serverlessrepo", config=client_config)
                     self._make_service_call_with_retry(service_call, app_id, semver, key, logical_id)  # type: ignore[no-untyped-call]
                 except InvalidResourceException as e:
                     # Catch all InvalidResourceExceptions, raise those in the before_resource_transform target.
@@ -150,8 +165,8 @@ class ServerlessAppPlugin(BasePlugin):
 
     def _get_intrinsic_resolvers(self, mappings):  # type: ignore[no-untyped-def]
         return [
-            IntrinsicsResolver(self._parameters),  # type: ignore[no-untyped-call]
-            IntrinsicsResolver(mappings, {FindInMapAction.intrinsic_name: FindInMapAction()}),  # type: ignore[no-untyped-call, no-untyped-call]
+            IntrinsicsResolver(self._parameters),
+            IntrinsicsResolver(mappings, {FindInMapAction.intrinsic_name: FindInMapAction()}),
         ]
 
     def _resolve_location_value(self, value, intrinsic_resolvers):  # type: ignore[no-untyped-def]
@@ -260,9 +275,7 @@ class ServerlessAppPlugin(BasePlugin):
         )
 
         app_id = resource_properties[self.LOCATION_KEY].get(self.APPLICATION_ID_KEY)
-
-        if not app_id:
-            raise InvalidResourceException(logical_id, "Property 'ApplicationId' cannot be blank.")
+        app_id = sam_expect(app_id, logical_id, "ApplicationId").to_not_be_none()
 
         if isinstance(app_id, dict):
             raise InvalidResourceException(
@@ -283,7 +296,7 @@ class ServerlessAppPlugin(BasePlugin):
                 "and Ref intrinsic functions are supported.",
             )
 
-        key = (app_id, semver)
+        key = self._make_app_key(app_id, semver)
 
         # Throw any resource exceptions saved from the before_transform_template event
         if isinstance(self._applications[key], InvalidResourceException):
