@@ -3,8 +3,8 @@ class TemplateNotFoundException(Exception):
     Exception raised when a template with given name is not found
     """
 
-    def __init__(self, template_name):
-        super(TemplateNotFoundException, self).__init__("Template with name '{}' is not found".format(template_name))
+    def __init__(self, template_name):  # type: ignore[no-untyped-def]
+        super(TemplateNotFoundException, self).__init__(f"Template with name '{template_name}' is not found")
 
 
 class InsufficientParameterValues(Exception):
@@ -12,7 +12,7 @@ class InsufficientParameterValues(Exception):
     Exception raised when not every parameter in the template is given a value.
     """
 
-    def __init__(self, message):
+    def __init__(self, message):  # type: ignore[no-untyped-def]
         super(InsufficientParameterValues, self).__init__(message)
 
 
@@ -21,5 +21,5 @@ class InvalidParameterValues(Exception):
     Exception raised when parameter values passed to this template is invalid
     """
 
-    def __init__(self, message):
+    def __init__(self, message):  # type: ignore[no-untyped-def]
         super(InvalidParameterValues, self).__init__(message)
