@@ -2,7 +2,7 @@ import copy
 
 from samtranslator.metrics.method_decorator import MetricsMethodWrapperSingleton
 from samtranslator.metrics.metrics import DummyMetricsPublisher, Metrics
-
+from typing import Dict, Any, Optional, List, Tuple
 from samtranslator.feature_toggle.feature_toggle import (
     FeatureToggle,
     FeatureToggleDefaultConfigProvider,
@@ -36,7 +36,7 @@ from samtranslator.model.eventsources.push import Api
 class Translator:
     """Translates SAM templates into CloudFormation templates"""
 
-    def __init__(self, managed_policy_map, sam_parser, plugins=None, boto_session=None, metrics=None):
+    def __init__(self, managed_policy_map, sam_parser, plugins=None, boto_session=None, metrics=None):  # type: ignore[no-untyped-def]
         """
         :param dict managed_policy_map: Map of managed policy names to the ARNs
         :param sam_parser: Instance of a SAM Parser
@@ -48,14 +48,16 @@ class Translator:
         self.sam_parser = sam_parser
         self.feature_toggle = None
         self.boto_session = boto_session
-        self.metrics = metrics if metrics else Metrics("ServerlessTransform", DummyMetricsPublisher())
-        MetricsMethodWrapperSingleton.set_instance(self.metrics)
+        self.metrics = metrics if metrics else Metrics("ServerlessTransform", DummyMetricsPublisher())  # type: ignore[no-untyped-call, no-untyped-call]
+        MetricsMethodWrapperSingleton.set_instance(self.metrics)  # type: ignore[no-untyped-call]
         self._translated_resouce_mapping = {}
 
         if self.boto_session:
             ArnGenerator.BOTO_SESSION_REGION_NAME = self.boto_session.region_name
 
-    def _get_function_names(self, resource_dict, intrinsics_resolver):
+    def _get_function_names(
+        self, resource_dict: Dict[str, Any], intrinsics_resolver: IntrinsicsResolver
+    ) -> Dict[str, str]:
         """
         :param resource_dict: AWS::Serverless::Function resource is provided as input
         :param intrinsics_resolver: to resolve intrinsics for function_name
@@ -72,18 +74,25 @@ class Translator:
                 if item.get("Type") == "Api" and item_properties.get("RestApiId"):
                     rest_api = item_properties.get("RestApiId")
                     api_name = Api.get_rest_api_id_string(rest_api)
-                    if isinstance(api_name, str):
-                        resource_dict_copy = copy.deepcopy(resource_dict)
-                        function_name = intrinsics_resolver.resolve_parameter_refs(
-                            resource_dict_copy.get("Properties").get("FunctionName")
-                        )
-                        if function_name:
-                            self.function_names[api_name] = str(self.function_names.get(api_name, "")) + str(
-                                function_name
-                            )
+                    if not isinstance(api_name, str):
+                        continue
+                    raw_function_name = resource_dict.get("Properties", {}).get("FunctionName")
+                    resolved_function_name = intrinsics_resolver.resolve_parameter_refs(
+                        copy.deepcopy(raw_function_name)
+                    )
+                    if not resolved_function_name:
+                        continue
+                    self.function_names.setdefault(api_name, "")
+                    self.function_names[api_name] += str(resolved_function_name)
         return self.function_names
 
-    def translate(self, sam_template, parameter_values, feature_toggle=None, passthrough_metadata=False):
+    def translate(
+        self,
+        sam_template: Dict[str, Any],
+        parameter_values: Dict[Any, Any],
+        feature_toggle: Optional[FeatureToggle] = None,
+        passthrough_metadata: Optional[bool] = False,
+    ) -> Dict[str, Any]:
         """Loads the SAM resources from the given SAM manifest, replaces them with their corresponding
         CloudFormation resources, and returns the resulting CloudFormation template.
 
@@ -101,13 +110,13 @@ class Translator:
         self.feature_toggle = (
             feature_toggle
             if feature_toggle
-            else FeatureToggle(FeatureToggleDefaultConfigProvider(), stage=None, account_id=None, region=None)
+            else FeatureToggle(FeatureToggleDefaultConfigProvider(), stage=None, account_id=None, region=None)  # type: ignore[no-untyped-call, no-untyped-call]
         )
-        self.function_names = dict()
-        self.redeploy_restapi_parameters = dict()
+        self.function_names: Dict[Any, Any] = {}
+        self.redeploy_restapi_parameters = {}
         sam_parameter_values = SamParameterValues(parameter_values)
         sam_parameter_values.add_default_parameter_values(sam_template)
-        sam_parameter_values.add_pseudo_parameter_values(self.boto_session)
+        sam_parameter_values.add_pseudo_parameter_values(self.boto_session)  # type: ignore[no-untyped-call]
         parameter_values = sam_parameter_values.parameter_values
         # Create & Install plugins
         sam_plugins = prepare_plugins(self.plugins, parameter_values)
@@ -130,7 +139,7 @@ class Translator:
         shared_api_usage_plan = SharedApiUsagePlan()
         document_errors = []
         changed_logical_ids = {}
-        route53_record_set_groups = {}
+        route53_record_set_groups: Dict[Any, Any] = {}
         for logical_id, resource_dict in self._get_resources_to_iterate(sam_template, macro_resolver):
             try:
                 macro = macro_resolver.resolve_resource_type(resource_dict).from_dict(
@@ -162,7 +171,7 @@ class Translator:
 
                 del template["Resources"][logical_id]
                 for resource in translated:
-                    if verify_unique_logical_id(resource, sam_template["Resources"]):
+                    if verify_unique_logical_id(resource, sam_template["Resources"]):  # type: ignore[no-untyped-call]
                         # For each generated resource, pass through existing metadata that may exist on the original SAM resource.
                         _r = resource.to_dict()
                         if resource_dict.get("Metadata") and passthrough_metadata:
@@ -174,17 +183,17 @@ class Translator:
                             DuplicateLogicalIdException(logical_id, resource.logical_id, resource.resource_type)
                         )
             except (InvalidResourceException, InvalidEventException, InvalidTemplateException) as e:
-                document_errors.append(e)
+                document_errors.append(e)  # type: ignore[arg-type]
 
-        if deployment_preference_collection.any_enabled():
-            template["Resources"].update(deployment_preference_collection.get_codedeploy_application().to_dict())
+        if deployment_preference_collection.any_enabled():  # type: ignore[no-untyped-call]
+            template["Resources"].update(deployment_preference_collection.get_codedeploy_application().to_dict())  # type: ignore[no-untyped-call]
             if deployment_preference_collection.needs_resource_condition():
                 new_conditions = deployment_preference_collection.create_aggregate_deployment_condition()
                 if new_conditions:
-                    template.get("Conditions").update(new_conditions)
+                    template.get("Conditions", {}).update(new_conditions)
 
-            if not deployment_preference_collection.can_skip_service_role():
-                template["Resources"].update(deployment_preference_collection.get_codedeploy_iam_role().to_dict())
+            if not deployment_preference_collection.can_skip_service_role():  # type: ignore[no-untyped-call]
+                template["Resources"].update(deployment_preference_collection.get_codedeploy_iam_role().to_dict())  # type: ignore[no-untyped-call]
 
             for logical_id in deployment_preference_collection.enabled_logical_ids():
                 try:
@@ -192,13 +201,13 @@ class Translator:
                         deployment_preference_collection.deployment_group(logical_id).to_dict()
                     )
                 except InvalidResourceException as e:
-                    document_errors.append(e)
+                    document_errors.append(e)  # type: ignore[arg-type]
 
         # Run the after-transform plugin target
         try:
             sam_plugins.act(LifeCycleEvents.after_transform_template, template)
         except (InvalidDocumentException, InvalidResourceException, InvalidTemplateException) as e:
-            document_errors.append(e)
+            document_errors.append(e)  # type: ignore[arg-type]
 
         # Cleanup
         if "Transform" in template:
@@ -208,11 +217,12 @@ class Translator:
             template = intrinsics_resolver.resolve_sam_resource_id_refs(template, changed_logical_ids)
             template = intrinsics_resolver.resolve_sam_resource_refs(template, supported_resource_refs)
             return template
-        else:
-            raise InvalidDocumentException(document_errors)
+        raise InvalidDocumentException(document_errors)
 
     # private methods
-    def _get_resources_to_iterate(self, sam_template, macro_resolver):
+    def _get_resources_to_iterate(
+        self, sam_template: Dict[str, Any], macro_resolver: ResourceTypeResolver
+    ) -> List[Tuple[str, Dict[str, Any]]]:
         """
         Returns a list of resources to iterate, order them based on the following order:
 
@@ -244,7 +254,7 @@ class Translator:
             # Skip over the resource if it is not a SAM defined Resource
             if not macro_resolver.can_resolve(resource):
                 continue
-            elif resource["Type"] == "AWS::Serverless::Function":
+            if resource["Type"] == "AWS::Serverless::Function":
                 functions.append(data)
             elif resource["Type"] == "AWS::Serverless::StateMachine":
                 statemachines.append(data)
@@ -258,7 +268,7 @@ class Translator:
         return functions + statemachines + apis + others + connectors
 
 
-def prepare_plugins(plugins, parameters=None):
+def prepare_plugins(plugins: List[Any], parameters: Optional[Dict[str, Any]] = None) -> SamPlugins:
     """
     Creates & returns a plugins object with the given list of plugins installed. In addition to the given plugins,
     we will also install a few "required" plugins that are necessary to provide complete support for SAM template spec.
@@ -272,8 +282,8 @@ def prepare_plugins(plugins, parameters=None):
         parameters = {}
     required_plugins = [
         DefaultDefinitionBodyPlugin(),
-        make_implicit_rest_api_plugin(),
-        make_implicit_http_api_plugin(),
+        make_implicit_rest_api_plugin(),  # type: ignore[no-untyped-call]
+        make_implicit_http_api_plugin(),  # type: ignore[no-untyped-call]
         GlobalsPlugin(),
         make_policy_template_for_function_plugin(),
     ]
@@ -282,28 +292,28 @@ def prepare_plugins(plugins, parameters=None):
 
     # If a ServerlessAppPlugin does not yet exist, create one and add to the beginning of the required plugins list.
     if not any(isinstance(plugin, ServerlessAppPlugin) for plugin in plugins):
-        required_plugins.insert(0, ServerlessAppPlugin(parameters=parameters))
+        required_plugins.insert(0, ServerlessAppPlugin(parameters=parameters))  # type: ignore[no-untyped-call]
 
     # Execute customer's plugins first before running SAM plugins. It is very important to retain this order because
     # other plugins will be dependent on this ordering.
     return SamPlugins(plugins + required_plugins)
 
 
-def make_implicit_rest_api_plugin():
+def make_implicit_rest_api_plugin():  # type: ignore[no-untyped-def]
     # This is necessary to prevent a circular dependency on imports when loading package
     from samtranslator.plugins.api.implicit_rest_api_plugin import ImplicitRestApiPlugin
 
     return ImplicitRestApiPlugin()
 
 
-def make_implicit_http_api_plugin():
+def make_implicit_http_api_plugin():  # type: ignore[no-untyped-def]
     # This is necessary to prevent a circular dependency on imports when loading package
     from samtranslator.plugins.api.implicit_http_api_plugin import ImplicitHttpApiPlugin
 
     return ImplicitHttpApiPlugin()
 
 
-def make_policy_template_for_function_plugin():
+def make_policy_template_for_function_plugin() -> PolicyTemplatesForResourcePlugin:
     """
     Constructs an instance of policy templates processing plugin using default policy templates JSON data
 
@@ -312,4 +322,4 @@ def make_policy_template_for_function_plugin():
 
     policy_templates = PolicyTemplatesProcessor.get_default_policy_templates_json()
     processor = PolicyTemplatesProcessor(policy_templates)
-    return PolicyTemplatesForResourcePlugin(processor)
+    return PolicyTemplatesForResourcePlugin(processor)  # type: ignore[no-untyped-call]
